

def public_key():
    return (b'-----BEGIN PUBLIC KEY-----\n'
            b'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAz'
            b'/3AuYbBdy83TjnmqE0FmGRBa64HtbGkKS0U4nHmE2yT0B'
            b'DxLywPGUSlf/od9ASJJiul+oxNHqO4sY/NguDD9NOI0FB'
            b'm1Mg73wT5PNmv3EPiwRpvsF2IeF0ZQDDPiQ/66dLzEZSs'
            b'AB6oDomJkE1KFCfdwrgd+LxYYG7+pmH+gxTkUDwSwv91M'
            b'ET8Huo+LIRj3StoDgfvBnv3IKNgSCkmgVtmIcB25tJMVw'
            b'n5bJAbuMkMnoOJMvHcAc53wpNl1lWV1wx4i6+69nnyxi8'
            b'Ba+li/zEVW5pOtkP0EYkKCyaeiwdwZnIea9iFhHXHNWdT'
            b'JWex9kRaD4oHWpcj1Oem3b2/twIDAQAB\n'
            b'-----END PUBLIC KEY-----')
