IDENTITY_TOKEN_URL = 'https://identity.api.rackspacecloud.com/v2.0/tokens'
ENCRYPT_KEY_URL = ('http://www.rackspace.com/knowledge_center/'
                   'article/generating-your-encrypted-key-in-cloud-backup')
